Module: Datastar::Consts
- Defined in:
- lib/datastar/consts.rb
Defined Under Namespace
Modules: FragmentMergeMode
Constant Summary collapse
- DATASTAR_KEY =
'datastar'- VERSION =
'1.0.0-beta.5'- DEFAULT_FRAGMENTS_SETTLE_DURATION =
The default duration for settling during fragment merges. Allows for CSS transitions to complete.
300- DEFAULT_SSE_RETRY_DURATION =
The default duration for retrying SSE on connection reset. This is part of the underlying retry mechanism of SSE.
1000- DEFAULT_FRAGMENTS_USE_VIEW_TRANSITIONS =
Should fragments be merged using the ViewTransition API?
false- DEFAULT_MERGE_SIGNALS_ONLY_IF_MISSING =
Should a given set of signals merge if they are missing?
false- DEFAULT_EXECUTE_SCRIPT_AUTO_REMOVE =
Should script element remove itself after execution?
true- DEFAULT_EXECUTE_SCRIPT_ATTRIBUTES =
The default attributes for <script/> element use when executing scripts. It is a set of key-value pairs delimited by a newline \n character.}
'type module'- DEFAULT_FRAGMENT_MERGE_MODE =
The mode in which a fragment is merged into the DOM.
FragmentMergeMode::MORPH
- SELECTOR_DATALINE_LITERAL =
Dataline literals.
'selector'- MERGE_MODE_DATALINE_LITERAL =
'mergeMode'- SETTLE_DURATION_DATALINE_LITERAL =
'settleDuration'- FRAGMENTS_DATALINE_LITERAL =
'fragments'- USE_VIEW_TRANSITION_DATALINE_LITERAL =
'useViewTransition'- SIGNALS_DATALINE_LITERAL =
'signals'- ONLY_IF_MISSING_DATALINE_LITERAL =
'onlyIfMissing'- PATHS_DATALINE_LITERAL =
'paths'- SCRIPT_DATALINE_LITERAL =
'script'- ATTRIBUTES_DATALINE_LITERAL =
'attributes'- AUTO_REMOVE_DATALINE_LITERAL =
'autoRemove'